<?php

namespace App\Http\Controllers;

use App\Http\Models\Complaint;
use App\Http\Models\ComplaintStatus;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class AdminController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        //$this->middleware('auth');
        $this->middleware('admin');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        $general['title']="Dashboard";
        $userid=Auth::user()->id;
        $all_complaints=Complaint::all()->count();
        $assignedSupervisor=Complaint::whereIn('status_id',[2,7])
            ->count();
        $assigned_to_vendor=Complaint::whereIn('status_id',[4,8])
            ->count();
        $assigned_to_company=Complaint::Where('status_id', '=', '3')
            ->count();    
        $notAssigned=Complaint::where('status_id', '=', '1')
            ->count();
        $all_complaints=Complaint::all()->count();
        $pending=Complaint::where('status_id', '!=', '5')->count();
        $all_company_complaints=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) {
                $q->whereIn('status_id',[3]);
            })
            ->count();
            $all_vendor_complaints=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) {
                $q->whereIn('status_id',[4,8]);
            })
            ->count();         
            $complaints_added_today=Complaint::wheredate('created_at','=',date('Y-m-d'))
            ->count();
        $complaints_completed_today=Complaint::where('status_id', '=', '5')
            ->wheredate('updated_at','=',date('Y-m-d'))
            ->count(); 
        $completed=Complaint::where('status_id', '=', '5')
            ->count();
            $non_updated_24_hours=Complaint::where('status_id', '!=', '5')
            ->where('updated_at','<',Carbon::now()->subHours(15))
            ->count();
            $non_updated_48_hours=Complaint::where('status_id', '!=', '5')
            ->where('updated_at','<',Carbon::now()->subHours(48))
            ->count();
            $non_updated_72_hours=Complaint::where('status_id', '!=', '5')
            ->where('updated_at','<',Carbon::now()->subHours(72))
            ->count();
        $notResolved=Complaint::where('status_id', '!=', '5')
            ->where('created_at','<',Carbon::now()->subDays(15))
            ->count();
        $pending_7_days=Complaint::where('status_id', '!=', '5')
            ->where('created_at','<',Carbon::now()->subDays(7))
            ->count();    
        $special=Complaint::where('status_id', '=', '16')
            ->count();
            $reopened_pending=Complaint::where('status_id', '!=', '5')
                ->where('reopened','=',1)
                ->count();
        $reopened_completed=Complaint::where('status_id', '=', '5')
                ->where('reopened','=',1)
                ->count();
                
         $alertpending=Complaint::where('status_id', '!=', '5')->where('alert_complaint' , 1)->count();
         
        return view('admin.dashboard',compact("alertpending" , "general","pending","completed","all_complaints","all_company_complaints","all_vendor_complaints","assignedSupervisor","notResolved","special","assigned_to_vendor","assigned_to_company","notAssigned","non_updated_24_hours","non_updated_48_hours","non_updated_72_hours","complaints_completed_today","complaints_added_today","reopened_completed","reopened_pending","pending_7_days"));
    }


    function specialRequest(){
        $general['title']="Special Request";
        $general['msg']=" ";
        $userid = Auth::id();
        $complaints = Complaint::with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
            })
            ->where('status_id', '=', '16')
            ->get()->sortByDesc('id');

        return view('admin.specialRequest',compact("general","complaints"));
    }
    function specialRequestPost(Request $request){
        $id=$request->input('id');
        $complaint = Complaint::find($id);
        if ($complaint->status_id==16){
                $status = ComplaintStatus::where(['status_id' => 2, 'complaint_id' => $id])->first();
                $assignTo = $status->assign_to;
                $status_id = 17;
                //$comment = $request->input('comment');
                $second_status = $request->input('status');
                $cstatus = new ComplaintStatus([/*'comment' => $comment,*/ 'status_id' => $status_id, 'second_status_id' => $second_status, 'assign_to' => $assignTo]);
                $complaint = Complaint::find($id);
                $complaint->status_id = $status_id;
                $complaint->complaintstatus()->save($cstatus);

        }
        return redirect(route('admin.specialReqest'));
    }


    public function completed()
    {
        $general['title'] = "Completed Complaints";
        $general['msg'] = "Completed Complaints";
        $userid = Auth::id();
        $complaints = Complaint::where('status_id', '=', '5')->paginate(50);

        return view('admin.completed',compact("general","complaints"));
    }
    public function all()
    {
        $general['title'] = "All Complaints";
        $general['msg'] = "All Complaints";
        $userid = Auth::id();
        $complaints = Complaint::paginate(50);

        return view('admin.all',compact("general","complaints"));
    }



}
