<?php

namespace App\Http\Controllers;

use App\Http\Models\Company;
use Illuminate\Http\Request;

class CompanyController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $general['title']="All Companies";
        $general['msg']="Click on \"Edit\" to edit company";
        $companies=Company::all()->sortByDesc('id');
        return view('company.all_companies',compact("general","companies"));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $general['title']="Create Company";
        $general['msg']="";
        return view('company.create',compact("general"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'name' => 'required',
            'email' => 'required',
            'phone_no' => 'required',
            'ptcl_no' => 'required',
            'address' => 'required',
        ]);
        $company=new Company(
            $request->all()
        );
        $company->save();
        session()->put('companyCreated', 'Successfully Created');
        return redirect(route('admin.company.index'));
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function show(Company $company)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function edit(Company $company)
    {
        $general['title']="Edit Company";
        $general['msg']="";
        return view('company.edit',compact("general","company"));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Company $company)
    {
        $validatedData = $request->validate([
            'name' => 'required',
            'email' => 'required',
            'phone_no' => 'required',
            'ptcl_no' => 'required',
            'address' => 'required',
        ]);
        $company->fill($request->all());
        $company->save();
        session()->put('companyCreated', 'Successfully Created');
        return redirect(route('admin.company.index'));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function destroy(Company $company)
    {
        $company->delete();
        session()->put('companyCreated', 'Successfully Deleted');
    }
}
