<?php

namespace App\Http\Controllers;

use App\Http\Middleware\Superviser;
use App\Http\Models\Branch;
use App\Http\Models\Company;
use App\Http\Models\Status;
use App\Http\Models\Complaint;
use App\Http\Models\ComplaintStatus;
use App\Http\Models\ComplaintRemarks;
use App\Http\Models\ProductCategory;
use App\Http\Models\Salesman;
use App\Http\Models\User;
use App\Http\Models\Vendor;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use Helper;

class ComplaintController extends Controller
{
    public function __construct()
    {
        //$this->middleware(['admin','csr']);
        //$this->middleware('admin');
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        
        $general['title']="Dashboard";
        $pending=Complaint::where('status_id', '!=', '5')->count();
        $completed=Complaint::where('status_id', '=', '5')->count();
        $unassigned=Complaint::where('status_id', '=', '1')->count();
        $uncompleted_forty_hours=Complaint::where('status_id', '!=', '5')
                    ->where('created_at','<',Carbon::now()->subHours(48))
                    ->count();
        $non_updated_24_hours=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subHours(24))
        ->count();
        $non_updated_48_hours=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subHours(48))
        ->count();
        $non_updated_72_hours=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subHours(72))
        ->count();
        $seven_days_pending=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subDays(7))
        ->count();
        $complaints_added_today=Complaint::wheredate('created_at','=',date('Y-m-d'))
            ->count();
        $complaints_completed_today=Complaint::where('status_id', '=', '5')
            ->wheredate('updated_at','=',date('Y-m-d'))
            ->count();   
        $vendor_pending=Complaint::whereIn('status_id',[4,8])->count();
        $company_pending=Complaint::with (['complaintstatus'])
                    ->whereHas('complaintstatus',function($q) {
                        $q->where(['status_id'=>3]);
                    })
                    ->where('status_id', '!=', '5')
                    ->count(); 
        $all_vendor_complaints=Complaint::with (['complaintstatus'])
                ->whereHas('complaintstatus',function($q) {
                    $q->whereIn('status_id',[4,8]);
                })
                ->count();
        $all_company_complaints=Complaint::with (['complaintstatus'])
                ->whereHas('complaintstatus',function($q) {
                    $q->whereIn('status_id',[3]);
                })
                ->count();                            
        $company_assigned=Complaint::where('status_id', '=', '3')->count();
        $happy_call_pending=Complaint::where('status_id','5')->whereNull('feedback')->count(); // feedback not added and complaint completed
        $happy_call_done=Complaint::where('status_id','5')->whereNotNull('feedback')->count(); // feedback added and complaint completed
        $showroom_complaints=Complaint::where('complaint_type','showroom')->count();
        $reopened_pending=Complaint::where('status_id', '!=', '5')
        ->where('reopened','=',1)
        ->count();
        $reopened_completed=Complaint::where('status_id', '=', '5')
        ->where('reopened','=',1)
        ->count();
        
        $reassigned=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) {
           $q->whereNotIn('second_status_id' , [22,20]);
             $q->where('status_id' , 7);
            })
            ->where('status_id', '=', '7')
            ->count();
            $alertpending=Complaint::where('status_id', '!=', '5')->where('alert_complaint' , 1)->count();

        return view('csr.dashboard',compact("alertpending" , "reassigned" , "general","pending","completed","unassigned","showroom_complaints","happy_call_pending","happy_call_done","all_company_complaints","uncompleted_forty_hours","non_updated_24_hours","non_updated_48_hours","non_updated_72_hours","vendor_pending","all_vendor_complaints","company_assigned","seven_days_pending","company_pending","complaints_completed_today","complaints_added_today","reopened_pending","reopened_completed"));

        // $general['title'] = "Pending Complaints";
        // $general['msg'] = "Click on \"Assign to\" to assign Vendors and Companies";
        // $userid = Auth::id();
        // $complaints=Complaint::where('status_id',1)->get()->sortByDesc('id');
        // $vendors=Vendor::all()->sortByDesc('id');
        // $companies=Company::all()->sortByDesc('id');

        // return view('supervisor.pending',compact("general","complaints","companies","vendors"));
    }
    
     public function reassigned(Request $request)
    {
        
        $general['title'] = "Reassigned Complaints";
        $general['selected_record'] = "all_reassigned";
        $com=Complaint::query();
        if($request->has('pending_duration') && $request->input('pending_duration')=="more_than_48_hrs"){
            $com->where('updated_at','<',Carbon::now()->subHours(48));  // more than 48 hrs passed
            $general['selected_record'] = "more_than_48_hrs";
        }
        
        
    $complaints = $com->withAndWhereHas('complaintstatus', function ($q){
    $q->whereNotIn('second_status_id' , [22,20]);
     $q->where('status_id' , 7);
    })->where('status_id', '=', '7')->get()
    ->sortByDesc('id');
            
        
        return view('supervisor.reassigned',compact("general","complaints"));
    }
    
    public function search(Request $request)
    {
         if($request->ajax())
     {
      $output = '';
      $query = $request->get('query');
      
      if($query != '')
      {
       $data = Complaint::where('name', 'like', '%'.$query.'%')
         ->orWhere('phone_no', 'like', '%'.$query.'%')
         ->orWhere('address', 'like', '%'.$query.'%')
         ->orWhere('invoice_no', 'like', '%'.$query.'%')
         ->orWhere('complaint_type', 'like', '%'.$query.'%')
         ->orWhere('item_make', 'like', '%'.$query.'%')
         ->orWhere('serial_no', 'like', '%'.$query.'%')
         ->orWhere('model', 'like', '%'.$query.'%')
         ->orWhere('product', 'like', '%'.$query.'%')
         ->orWhere('problem', 'like', '%'.$query.'%')
         ->orWhere('created_at', 'like', '%'.$query.'%')
         ->orderBy('id', 'desc')
         ->get();
         
      }
      else
      {
       $data = Complaint::orderBy('id', 'desc')
         ->get();
      }
      $total_row = $data->count();
      $output .='<button onclick="return hideList()" style="color: red;float: right;background: transparent;border: none;font-size: 1.5rem;margin-top: -10px;">x</button>';
      $output .= ' <table  class="table table-bordered table-hover">
                            <thead>
                            <tr>
                            <th class="sorting">Name</th>
                            <th>Phone</th>
                            <th>Address</th>
                            <th>Type</th>
                            <th>Created At</th>
                            <th>Action</th>
                            </tr>
                            </thead>
                            <tbody>';
      if($total_row > 0)
      {
          
       foreach($data as $row)
       {
        $output .= '
      <tr>
         <td > '.$row->name.'</td>
         <td > '.$row->phone_no.'</td>
         <td > '.$row->address.'</td>
         <td > '.$row->complaint_type.'</td>
          <td > '.$row->created_at.'</td>
         <td ><a data-toggle="tooltip" title="" href="'.url('complaint/'.$row->id).'" data-original-title="View Detail"><span class="fa fa-eye"></span></a></td>
       </tr>
        ';
       }
      }
      else
      {
       $output = '
       <tr>
        <div>No Data Found</div>
     </tr>
       ';
      }
      
      $output .='</tbody>
                <tfoot>
                <tr>
                <th>Name</th>
                <th>Phone</th>
                <th>Address</th>
                <th>Type</th>
                <th>Created At</th>
                <th>Action</th>
                </tr>
                </tfoot>
                </table>';
      
      $data = array(
       'table_data'  => $output
      );

      echo json_encode($data);
     }
     
    }

    public function unassigned()
    
    {
        $general['title']="Unassigned  Complaints";
        $general['msg']="";
        $complaints=Complaint::where('status_id',1)->get()->sortByDesc('id');
        $supervisors=User::where('role_id',3)->get()->sortByDesc('id');
        $companies=Company::all()->sortByDesc('id');
        return view('complaint.new_complaints',compact("general","complaints","supervisors","companies"));
    }

    public function pending(){
        
        $general['title']="Pending Complaints";
       $general['msg']="";
       $complaints=Complaint::where('status_id','!=',5)->paginate(50);
       return view('complaint.general_complaints_temp',compact("general","complaints"));
   }
   
   function pendingAlertComplaint()
   {
        $general['title']="Pending Alert Complaints";
       $general['msg']="";
       $complaints=Complaint::where('status_id','!=',5)->where('alert_complaint' , 1)->paginate(50);
       return view('complaint.general_complaints_temp',compact("general","complaints"));
   }
   
   public function completed()
   {
        $general['title']="Completed Complaints";
        $general['msg']="";
        $complaints=Complaint::where('status_id','=',5)->paginate(50);
        return view('complaint.general_complaints_temp',compact("general","complaints"));
    }

    public function showroom_complaints()
   {
        $general['title']="Shwroom Complaints";
        $general['msg']="";
        $complaints=Complaint::where('complaint_type','showroom')->paginate(50);
        return view('complaint.general_complaints_temp',compact("general","complaints"));
    }

    public function complaintsAddedToday(){
        $general['title']="Complaints added today";
       $general['msg']="";
       $complaints=Complaint::wheredate('created_at','=',date('Y-m-d'))->paginate(50);
       return view('complaint.general_complaints_temp',compact("general","complaints"));
   }

   public function  complaintscompletedToday(){
         $general['title']="Complaints completed today";
         $general['msg']="";
          $complaints=Complaint::where('status_id', '=', '5')
          ->wheredate('updated_at','=',date('Y-m-d'))->paginate(50);
         return view('complaint.general_complaints_temp',compact("general","complaints"));
    } 
    /***
     * complaints which are reopened and pending
     */
    public function  reopenedpending(){
        $general['title']="Reopened pending complaints";
        $general['msg']="";
         $complaints=Complaint::where('status_id', '!=', '5')
         ->where('reopened','=',1)->paginate(50);
        return view('complaint.general_complaints_temp',compact("general","complaints"));
   } 
   /***
     * complaints which are reopened and completed
     */
    public function  reopenedcompleted(){
        $general['title']="Reopened pending complaints";
        $general['msg']="";
         $complaints=Complaint::where('status_id', '=', '5')
         ->where('reopened','=',1)->paginate(50);
        return view('complaint.general_complaints_temp',compact("general","complaints"));
   }    
    public function report(Request $request)
    {
        $where=[];
        if($request->has('model') && $request->filled('model'))
            $where['model']=$request->input('model');
        if($request->has('make') && $request->filled('make'))
            $where['item_make']=$request->input('make'); 
            
        if($request->has('product') && $request->filled('product'))
            $where['product']=$request->input('product'); 
            
        if($request->has('invoice_no') && $request->filled('invoice_no'))
            $where['invoice_no']=$request->input('invoice_no'); 
        /*if($request->has('complain_status') && $request->filled('complain_status'))
            $where['status_id']=$request->input('complain_status');*/
        if($request->has('complaint_no') && $request->filled('complaint_no'))
            $where['id']=$request->input('complaint_no');
        if($request->has('salesman') && $request->filled('salesman'))
            $where['salesman_id']=$request->input('salesman');
        if($request->has('branch') && $request->filled('branch'))
            $where['branch_id']=$request->input('branch');
        if($request->has('complaint_type') && $request->filled('complaint_type'))
            $where['complaint_type']=$request->input('complaint_type');   
            
        if($request->has('supervisor_status') && $request->filled('supervisor_status'))
            $where['status_id']=$request->input('supervisor_status'); 
            
        $general['title']="Report Section";
        $general['msg']="";
        //$complaints=Complaint::where($where)->get()->sortByDesc('id');
        $com=Complaint::query();
        $comp=$com->where($where);
       /* if($request->has('customer_status') && $request->filled('customer_status')){
            
            if($request->input('customer_status')=='1') //pending
            $comp->where('status_id', '<>', '5'); 
            else
            $comp->where('status_id','=',$request->input('customer_status'));
        }*/
        /*if($request->has('vendor_id') && $request->filled('vendor_id'))
           $comp->whereIn('status_id',[4,8]);*/

        if($request->has('date_from') && $request->filled('date_from') && $request->has('date_to') && $request->filled('date_to'))
          $comp=$com->whereBetween('created_at',[$request->input('date_from') ,$request->input('date_to') ]);

        if($request->has('no_of_days_passed') && $request->filled('no_of_days_passed')){
            $com->whereDate('created_at','=',Carbon::now()->subDays($request->input('no_of_days_passed'))->toDateString());
            //$com->where('created_ats','>=',Carbon::now()->subDays($request->input('no_of_days_passed')));
        }
        if($request->has('created_at') && $request->filled('created_at'))
            $com->whereDate('created_at',$request->input('created_at')); 

        if($request->has('phone_search') && $request->filled('phone_search')){
           // $where['phone_no']=$request->input('phone_search');
            $comp->where('phone_no', 'like', '%' . $request->input('phone_search') . '%');
        }
              
        if($request->has('name_search') && $request->filled('name_search')){
            $comp->where('name', 'like', '%' . $request->input('name_search') . '%');
        }
        if($request->has('problem') && $request->filled('problem')){
            $comp->where('problem', 'like', '%' . $request->input('problem') . '%');
        }
        if($request->has('dop') && $request->filled('dop')){
            $comp->whereDate('dop', '=',  $request->input('dop') );
        }
           
          

        if(Auth::user()->role->name == "vendor")
        {
            $userid = Auth::user()->vendor->id;
            $comp->with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->where('status_id', '=', '4');
            });
        }
        if(Auth::user()->role->name == "branch_incharge")
        {
            $user_id = Auth::user()->id;
            $comp->where('created_by',$user_id);
        }
        if($request->has('vendor_id') && $request->filled('vendor_id'))
        {
            
            $vendor_id = $request->input('vendor_id');
            $comp->with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($vendor_id) {
                $q->where(['assign_to' => $vendor_id]);
                //$q->wherein('status_id', '=', '4');
                $q->whereIn('status_id',['4','8']);
            });
        }
        if($request->has('company') && $request->filled('company'))
        {
            $company_id=$request->input('company') ;
            $comp->with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($company_id) {
                $q->where(['assign_to' => $company_id]);
                $q->where('status_id','=',3);
            });
        }
        if($request->has('complain_status') && $request->filled('complain_status')){
            $status=$request->input('complain_status');
            if($status=="1"){
                $comp->where('status_id','!=','5');
            } else{
                $com->where('status_id','=','5');
            }
        }
        $complaints=$comp->paginate(50);
      
        
        $models=ProductCategory::distinct()->get('model');
        $makes=Complaint::distinct()->get('item_make');
        $products=Complaint::distinct()->get('product');
        $statuses=Status::get();
        
        $vendors=Vendor::all()->sortByDesc('id');
        $supervisors=User::where('role_id',3)->get()->sortByDesc('id');
        $companies=Company::all()->sortByDesc('id');
        $salesmens=Salesman::all()->sortByDesc('id');
        $branches=Branch::all();
        return view('admin.report',compact("products" , "general","supervisors","models","complaints","makes","statuses","vendors","companies","salesmens","branches"));
    }

    public function search_model(Request $request){
        // echo $request->datefilter;
        // die();
        $general['title']="Report Section";
       // $general['msg']="Click on \"Assign to\" to assign Supervisor";
        $general['msg']="Reports";
        if($request->make == 'null' && $request->model != 'null'){
        $complaints=Complaint::where('model','=',$request->model)->get()->sortByDesc('id');
        }else if($request->make != 'null' && $request->model == 'null'){
            $complaints=Complaint::where('item_make','=',$request->make)->get()->sortByDesc('id');
        }
        
        else if($request->make != 'null' && $request->model != 'null'){
            $complaints=Complaint::where('model','=',$request->model)
            ->where('item_make',$request->make)
            ->get()->sortByDesc('id');
        }else if($request->make == 'null' && $request->model == 'null' && $request->phone_search != ''){
            $complaints=Complaint::where('phone_no','=',$request->phone_search)->get()->sortByDesc('id');
            
        }else if($request->make == 'null' && $request->model == 'null' && $request->phone_search == '' && $request->complain_status != 'null'){
            $complaints=Complaint::where('status_id','=',$request->complain_status)->get()->sortByDesc('id');
            
        }
        // $itemmake=ProductCategory::get('item_make');
        $models=ProductCategory::get('model');
        $makes=Complaint::get('item_make');

        return view('admin.report',compact("general","complaints","models","makes"));
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function assigned()
    {
        /*$complaints=Complaint::with (['complaint_statuses'])
            ->whereHas('complaint_statuses',function($q) {
            $q->where(['status_id'=>'2','assign_to'=>3]);
        })
            ->where('status_id', '=', '2')
            ->get();*/
        /*$complaints=ComplaintStatus::with(['complaint' => function($q) {
            $q->where('status_id','=','2');
        }])
            ->get();*/

        $general['title']="Assigned Complaints";
        $general['msg']="Assigned Complaints";
       // $complaints=Complaint::where('status_id','=',2)->get()->sortByDesc('id');
        $complaints=Complaint::whereIn('status_id',['2','3'])->get()->sortByDesc('id');
      
        return view('complaint.assigned_complaint',compact("general","complaints"));
    }

    public function assignedToVendor()
    {
        
        $general['title']="Assigned Complaints";
        $general['msg']="To Vendor";
        $complaints=Complaint::whereIn('status_id',[4,8])->paginate(50);
        return view('complaint.assigned_to_complaint',compact("general","complaints"));
    }

    public function assignedToSupervisor()
    {
        
        $general['title']="Assigned Complaints";
        $general['msg']="To Supervisor";
        $complaints=Complaint::whereIn('status_id',[2,7])->get()->sortByDesc('id');
        return view('complaint.assigned_to_complaint',compact("general","complaints"));
    }

    public function assignedToCompany(Request $request)
    {
        $where=[];
        if($request->has('complaint_no') && $request->filled('complaint_no'))
            $where['id']=$request->input('complaint_no');

        $general['title']="Assigned Complaints";
        $general['msg']="To Company";
        $com=Complaint::query();
        $com->where($where);

        if($request->has('company') && $request->filled('company'))
        {
            $company_id=$request->input('company') ;
            $com->with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($company_id) {
                $q->where(['assign_to' => $company_id]);
                $q->where('status_id','=',3);
            });
        }
        if($request->has('complain_status') && $request->filled('complain_status')){
            $status=$request->input('complain_status');
            if($status=="1"){
                $com->where('status_id','!=','5');
            } else{
                $com->where('status_id','=','5');
            }
        }
        $complaints=$com->where('status_id','=',3)->get()->sortByDesc('id');
        $companies=Company::all()->sortByDesc('id');
        return view('complaint.assigned_to_company',compact("general","complaints","companies"));
    }

    public function companyPendingComplaints(Request $request)
    {
        $where=[];
        $general['title']="Company Pending Complaints";
        $general['msg']="To Company";
        if($request->has('complaint_no') && $request->filled('complaint_no'))
           $where['id']=$request->input('complaint_no');
        $com=Complaint::query();
        $com->where($where);
        if($request->has('company') && $request->filled('company'))
        {
            $company_id=$request->input('company') ;
            $com->with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($company_id) {
                $q->where(['assign_to' => $company_id]);
                $q->where('status_id','=',3);
            });
        }
        if($request->has('complain_status') && $request->filled('complain_status')){
            $status=$request->input('complain_status');
            if($status=="1"){
                $com->where('status_id','!=','5');
            } else{
                $com->where('status_id','=','5');
            }
        }
        
        $complaints=$com->with (['complaintstatus'])
                    ->whereHas('complaintstatus',function($q) {
                        $q->where(['status_id'=>3]);
                    })
                    ->where('status_id', '!=', '5')->paginate(50);
        $companies=Company::all()->sortByDesc('id');
        return view('complaint.assigned_to_company',compact("general","complaints","companies"));
    }

    public function allcompanyComplaints(Request $request)
    {
        $general['title']="All Company Complaints";
        $general['msg']="To Company";
        $where=[];
        if($request->has('complaint_no') && $request->filled('complaint_no'))
           $where['id']=$request->input('complaint_no');

        
        $com=Complaint::query();
        $com->where($where);
        if($request->has('complain_status') && $request->filled('complain_status')){
            $status=$request->input('complain_status');
            if($status=="1"){
                $com->where('status_id','!=','5');
            } else{
                $com->where('status_id','=','5');
            }
        }
        if($request->has('company') && $request->filled('company'))
        {
            $company_id=$request->input('company') ;
            $com->with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($company_id) {
                $q->where(['assign_to' => $company_id]);
                $q->where('status_id','=',3);
            });
        }
        //$complaints=$com->get()->sortByDesc('id');
        $complaints=$com->with (['complaintstatus'])
                    ->whereHas('complaintstatus',function($q) {
                        $q->whereIn('status_id',[3]);
                    })
                    ->paginate(50);
        $companies=Company::all()->sortByDesc('id');            
        return view('complaint.assigned_to_company',compact("general","complaints","companies"));
    }


    public function allVendorComplaints()
    {
        $general['title']="All vendor Complaints";
        $general['msg']="To Vendor";
        $complaints=Complaint::with (['complaintstatus'])
                    ->whereHas('complaintstatus',function($q) {
                        $q->whereIn('status_id',[4,8]);
                    })
                    ->paginate(50);
        return view('complaint.assigned_to_complaint',compact("general","complaints"));
    }

    /***
     * those complaints which are completed and feeback not added
     */
    public function feedbackPending()
    {
        $general['title']="Feedback pending  Complaints";
        $general['msg']="";
        $complaints=Complaint::where('status_id','5')->whereNull('feedback')->paginate(50);  
        return view('complaint.general_complaints_temp',compact("general","complaints"));
        
        
    }

    public function feedbackDone()
    {
        $general['title']="Feedback Completed  Complaints";
        $general['msg']="";
        $complaints=Complaint::where('status_id','5')->whereNotNull('feedback')->paginate(50);  
        return view('complaint.general_complaints_temp',compact("general","complaints"));
    }
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $general['title']="Create Complaint";
        $general['msg']="";
        $products=ProductCategory::distinct()->get('product');
        $branches=Branch::all();
        $supervisors=User::where('role_id',3)->get()->sortByDesc('id');
        $companies=Company::all()->sortByDesc('id');
          $vendors=Vendor::all()->sortByDesc('id');
        return view('complaint.add_complaint',compact("general","products","branches","supervisors","companies" , 'vendors'));
    }
    public function ComplaintView($id){
        $general['title']="Complaint";
        $general['msg']="Click on ";
        $complaint = Complaint::find($id);
        return view('complaint.complaint_view',compact("general","complaint"));
    }
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        
    
        $complaint = new Complaint();
        $complaint->name=$request->input('name');
        $complaint->phone_no=$request->input('phone_no');
        $complaint->address=$request->input('address');
        $complaint->branch_id=$request->input('branch_id');
        $complaint->salesman_id=$request->input('salesman_id');
        $complaint->invoice_no=$request->input('invoice_no');
        $complaint->product = $request->input('product');
         $complaint->product = $request->input('product');
        
        if ($request->input('product')=="other" || $request->input('item_make')=="other") {
            $complaint->item_make = $request->input('brand_name');
            $complaint->model = $request->input('model_name');
        }else {
            $complaint->item_make = $request->input('item_make');
            if ($request->input('model')=="other") {
                $complaint->model = $request->input('model_name');
            }else{
                $complaint->model = $request->input('model');
            }
        }
        $complaint->serial_no=$request->input('serial_no');
        $complaint->dop=$request->input('dop');
        $complaint->dop_manual=$request->input('dop_manual');
        $complaint->problem=$request->input('problem');
        $complaint->created_by = Auth::id();
        $complaint->save();
        $assign_to_type=$request->input('assign_to_type');
        if(in_array($assign_to_type,['company','supervisor']))
        {
            $request->request->add(['new_id' => $complaint->id]);
            $this->assign_after_creation($request); // assign as well if during creation complaint is assigned 
        }
        
        
          if($request->has('vendor') && $request->filled('vendor'))
          {
              $status= new ComplaintStatus();
              $status->status_id=4;
              $status->assign_to=$request->vendor;
              $status->complaint_id=$complaint->id;
              $status->save();
          } 
                
        
        return redirect()->route("csr.complaint.create")->with('success', 'Successfully Created!');
    }

    /***
     * reopen complaint
     */
    public function reopen(Request $request)
    {
    
        $prev_complaint_id=$request->input('id');
        $check_if_opened_already=Complaint::where('status_id', '!=', '5')
        ->where('prev_complaint_id','=',$prev_complaint_id)
        ->count();
        if($check_if_opened_already){
            return response()->json(['status'=>'failure','msg'=>'Already opened and pending']);
        }
        $complaint = new Complaint();
        $complaint->name=$request->input('name');
        $complaint->phone_no=$request->input('phone_no');
        $complaint->address=$request->input('address');
        $complaint->branch_id=$request->input('branch_id');
        $complaint->salesman_id=$request->input('salesman_id');
        $complaint->invoice_no=$request->input('invoice_no');
        $complaint->product = $request->input('product');
        $complaint->reopened = 1;
        $complaint->prev_complaint_id = $prev_complaint_id;
        
        if ($request->input('product')=="other" || $request->input('item_make')=="other") {
            $complaint->item_make = $request->input('brand_name');
            $complaint->model = $request->input('model_name');
        }else {
            $complaint->item_make = $request->input('item_make');
            if ($request->input('model')=="other") {
                $complaint->model = $request->input('model_name');
            }else{
                $complaint->model = $request->input('model');
            }
        }
        $complaint->serial_no=$request->input('serial_no');
        $complaint->dop=$request->input('dop');
        $complaint->dop_manual=$request->input('dop_manual');
        $complaint->problem=$request->input('problem');
        $complaint->created_by = Auth::id();
        $complaint->save();
        $assign_to_type=$request->input('assign_to_type');
        if(in_array($assign_to_type,['company','supervisor']))
        {
            $request->request->add(['new_id' => $complaint->id]);
            $this->assign_after_creation($request); // assign as well if during creation complaint is assigned 
        }
        return response()->json(['status'=>'success','reopened'=>'yes','msg'=>'Complaint Reopened']);
    }
    private function assign_after_creation($request)
    {
       // dd($request->all());
        $complaint=Complaint::find($request->input('new_id'));
        $assign_to_type=$request->input('assign_to_type');
        if(!in_array($assign_to_type,['company','supervisor']))
        {
            return ['status'=>'failure','msg'=>'Choose either supervisor or Company'];
        }
        if ($complaint){
            $status= new ComplaintStatus();
            $status->complaint_id=$complaint->id;
            if($request->input('assign_to_type')=="company")
            {
                $status->assign_to=$request->input('company_id');
                $status->status_id=3; // company status
            }
            else {
                $status->assign_to=$request->input('supervisor_id');
                $status->status_id=2;  // supervisor
            }
            $status->csr_remarks=$request->input('csr_remarks');
            $status->save();
             return ['status'=>'success','msg'=>'assigned'];
        }
        return ['status'=>'failure','msg'=>'Failed to assign'];

    }
    /**
     * Display the specified resource.
     *
     * @param  \App\Complaint  $complaint
     * @return \Illuminate\Http\Response
     */
    public function show(Complaint $complaint)
    {
        if($complaint->id && $complaint->view_status == 0 )
        {
           Complaint::where('id', $complaint->id)
            ->update(['view_status' => '1']); 
        }
        
            
        $general['title']="Complaint Details";
        $general['msg']="";
        return view('complaint.view',compact('complaint','general'));
    }
    
    function alertComplaint(Request $request)
    {
        $complaint_id = $request->input('complaint_id');
        Complaint::where('id', $complaint_id )->update(['alert_complaint' => '1']); 
        echo 'added';
    }

    public function updateCsrComment(Request $request){
        $pk_id=$request->input('table_pk_id');
        $comment=$request->input('comment');
        if($pk_id){
            $model=ComplaintStatus::find($pk_id);
            $model->csr_remarks=$comment;
            $model->save();
            //$model->update();
            return;
            ComplaintStatus::where('id',$pk_id)->update(['csr_remarks'=>$comment]);
            /*\DB::table('complaint_statuses')
            ->where('id', $pk_id)
            ->update(['csr_remarks' => $comment]);*/
        }
        return;
    }
    /****
     * csr add feedback
     */
    public function addFeedback(Request $request){
        $pk_id=$request->input('table_pk_id');
        $comment=$request->input('comment');
        if($pk_id){
            $model=Complaint::find($pk_id);
            $model->feedback=$comment;
            $model->save();
        }
        return;
    }
    /****
     * update status 
     */
    public function changestatus(Request $request)
    {
    
        $complaint=Complaint::find($request->input('id'));
        if ($complaint){
                $status= new ComplaintStatus();
                $status->status_id=$request->input('status_id');
                $status->comment=$request->input('comment');
                $status->complaint_id=$complaint->id;
                $status->save();

            session()->flash('success', 'Updated');
            return response()->json(['status'=>'success','msg'=>'Updated']);
        }
        return response()->json(['status'=>'failure','msg'=>'Failed to update']);

    }
    /****
     * add remarks 
     */
    public function addRemarks(Request $request)
    {
        
        $complaint=Complaint::find($request->input('id'));
        if ($complaint){
               $remarks=new ComplaintRemarks();
                $remarks->remarks=$request->input('comment');
                $remarks->complaint_id=$complaint->id;
                $remarks->created_by=Auth::User()->id;
                $remarks->save();
            return response()->json(['status'=>'success','msg'=>'Added']);
        }
        return response()->json(['status'=>'failure','msg'=>'Failed to add']);

    }

    /**
     * 
     */
    public function addCompanyComplaintNo(Request $request)
    {       
        $complaint=Complaint::find($request->input('id'));
        if ($complaint){
            $complaint->company_assigned_complain_no = $request->input('complaint_no');
            $complaint->save();
            return response()->json(['status'=>'success','msg'=>'Added']);
        }
        return response()->json(['status'=>'failure','msg'=>'Failed to add']);
    }

    /***
     * reminder 48 hours passed on report and did not taken action
     */
    public function fortyEightHoursPassed(Request $request)
    {
        $general['title']="Not completed last 48 hours";
        $where=[];
        $com=Complaint::query();
        if($request->has('record_type') && $request->input('record_type')=="not_assigned"){
            $comp=$com->where('status_id',1); 
            $general['selected_record']="not_assigned";
        }elseif($request->has('record_type') && $request->input('record_type')=="not_completed"){
            $comp=$com->whereNotIn('status_id',[5]);
            $general['selected_record']="not_completed";
        }else{
            $comp=$com->where('status_id',1);
            $general['selected_record']="not_assigned"; 
        }
           
        
        $comp=$com->where('created_at','<',Carbon::now()->subHours(48));  // more than 48 hrs passed
        $complaints=$comp->get()->sortByDesc('id');
        $supervisors=User::where('role_id',3)->get()->sortByDesc('id');
        $companies=Company::all()->sortByDesc('id');
        return view('complaint.unchanged_last_48hrs',compact('general','complaints','supervisors','companies'));
    }
    /***
     * reminder 48 hours passed on report and did not taken action overall with put filter
     */
    public function fortyEightHoursPassedNotCompleted(Request $request)
    {
        $general['title']="Not completed last 48 hours";
        $where=[];
        $com=Complaint::query();
        $comp=$com->where('created_at','<',Carbon::now()->subHours(48));  // more than 48 hrs passed
        $complaints=$comp->get()->sortByDesc('id');
        return view('complaint.general_complaints_temp',compact('general','complaints'));
    }

    /****
     * not updated since  hrs
     */
    public function notUpdatedfromhrs($hrs=24)
    {
        $general['title']="Not updated last $hrs hours";
        $complaints=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subHours($hrs))
        ->paginate(50);
        return view('complaint.general_complaints_temp',compact('general','complaints'));
        
    }
    
     /****
     * passed 7 days and not completed
     */
    public function uncompletedSevenDays()
    {
        $general['title']="Not Completed 7 days passed";
        $complaints=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subDays(7))->paginate(50);
        return view('complaint.general_complaints_temp',compact('general','complaints'));
        
    }
    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Complaint  $complaint
     * @return \Illuminate\Http\Response
     */
    public function edit(Complaint $complaint,$reopenning=null)
    {
       // dd($complaint);
        $general['title']= $reopenning ? "Reopenning Complaint":"Update Complaint";
        $general['msg']="";
        $products=ProductCategory::distinct()->get('product');
        $branches=Branch::all();
        $supervisors=User::where('role_id',3)->get()->sortByDesc('id');
        $companies=Company::all()->sortByDesc('id');
        $salesmen=Salesman::where(['branch_id'=>$complaint->branch_id])->get();
        $brands=ProductCategory::where('product',$complaint->product)->distinct()->get('brand');
        $models=ProductCategory::where(['product'=>$complaint->product,'brand'=>$complaint->item_make])->distinct()->get('model');
        $update_type=$reopenning ? "Reopen":"Update"; 
        return view('complaint.update_complaint',compact("general","brands","models","salesmen","products","branches","supervisors","companies","complaint","update_type"));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Complaint  $complaint
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
       
        //echo "<pre>";
       // print_r($status); die();
        
       // print_r($_POST); die();
        $complaint = Complaint::find($request->input('id'));
        $complaint->name=$request->input('name');
        $complaint->phone_no=$request->input('phone_no');
        $complaint->address=$request->input('address');
        $complaint->branch_id=$request->input('branch_id');
        $complaint->salesman_id=$request->input('salesman_id');
        $complaint->invoice_no=$request->input('invoice_no');
        $complaint->product = $request->input('product');
        if ($request->input('product')=="other" || $request->input('item_make')=="other") {
            $complaint->item_make = $request->input('brand_name');
            $complaint->model = $request->input('model_name');
        }else {
            $complaint->item_make = $request->input('item_make');
            if ($request->input('model')=="other") {
                $complaint->model = $request->input('model_name');
            }else{
                $complaint->model = $request->input('model');
            }
        }
        $complaint->serial_no=$request->input('serial_no');
        $complaint->dop=$request->input('dop');
        $complaint->dop_manual=$request->input('dop_manual');
        $complaint->problem=$request->input('problem');
        if($request->input('assign_to_type')=="company")
            $complaint->status_id=3; // company status
        elseif($request->input('assign_to_type')=="supervisor")
            $complaint->status_id=2;  // supervisor

        $complaint->created_by = Auth::id();
        if($complaint->update()){
            $assign_to_type=$request->input('assign_to_type');
            if(in_array($assign_to_type,['company','supervisor']))
            {
                $request->request->add(['id' => $complaint->id]);
                $this->update_status_after_edit_complaint($request); 
            }
            return response()->json(['status'=>'success','msg'=>'Updated']);
        }
        return response()->json(['status'=>'failure','msg'=>'Failed to update']);
        
        
    }
    /***
     * after update complaint 
     */
    private function update_status_after_edit_complaint($request)
    {
       // dd($request->all());
       // $complaint=Complaint::find($request->input('id'));
        $assign_to_type=$request->input('assign_to_type');
        if(!in_array($assign_to_type,['company','supervisor']))
        {
            return ['status'=>'failure','msg'=>'Choose either supervisor or Company'];
        }
            $status= ComplaintStatus::where('complaint_id','=',$request->input('id'))->first();
            if(!$status){
                $status= new ComplaintStatus();
                $status->complaint_id=$request->input('id');
            }
              

            if($request->input('assign_to_type')=="company")
            {
                $status->assign_to=$request->input('company_id');
                $status->status_id=3; // company status
            }
            else {
                $status->assign_to=$request->input('supervisor_id');
                $status->status_id=2;  // supervisor
            }
            $status->csr_remarks=$request->input('csr_remarks');
            if($status->save())
             return ['status'=>'success','msg'=>'updated'];
        
        return ['status'=>'failure','msg'=>'Failed to assign'];

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Complaint  $complaint
     * @return \Illuminate\Http\Response
     */
    public function destroy(Complaint $complaint)
    {
        //
    }
    public function assign(Request $request)
    {
        
        $complaint=Complaint::find($request->input('id'));
        $assign_to_type=$request->input('assign_to_type');
        if(!in_array($assign_to_type,['company','supervisor']))
        {
            return ['status'=>'failure','msg'=>'Choose either supervisor or Company'];
        }
        if ($complaint){
            $status= new ComplaintStatus();
            $status->complaint_id=$complaint->id;
            if($request->input('assign_to_type')=="company")
            {
                $status->assign_to=$request->input('company_id');
                $status->status_id=3; // company status
            }
            else {
                $status->assign_to=$request->input('supervisor_id');
                $status->status_id=2;  // supervisor
            }
            $status->csr_remarks=$request->input('csr_remarks');
            
            $status->save();
            
            Complaint::where('id' , $complaint->id)->update([
            'company_complaint_id' => $request->input('company_complaint_id'),
            ]);
            
            session()->put('assigned', 'Successfully Assigned');
             return ['status'=>'success','msg'=>'assigned'];
        }
        return ['status'=>'failure','msg'=>'Failed to assign'];

    }

    function vendor_complaints(){
        $general['title']="Your Complaints";
        $general['msg']="Click on \"Add Status\" to change Status";
        //$complaints=Complaint::where('vendor_id',Auth::user()->vendor->id)->get();

        $userid = Auth::user()->vendor->id;
        $complaints = Complaint::with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->where('status_id', '=', '4');
            })
            ->where('status_id', '=', '3')->orWhere('status_id', '=', '4')
            ->get()->sortByDesc('id');

        return view('complaint.assigned_complaint',compact("general","complaints"));
    }

    function getBrand($product){
        $products=ProductCategory::where('product',$product)->distinct()->get('brand');
        return json_encode($products);
    }
    function getModel($product,$brand){
        $products=ProductCategory::where(['product'=>$product,'brand'=>$brand])->distinct()->get('model');
        return json_encode($products);
    }
    function getSalesman($id){
        $salesmen=Salesman::where(['branch_id'=>$id])->get();
        return json_encode($salesmen);
    }
    public function sendSms($mobile,$complaintID)
    {
        return;
        /*$mobile=str_replace('+920',"0",$mobile);
        $mobile=str_replace('+92',"0",$mobile);
        $message ="Dear Customer, Your Complain #'+$complaintID+' has been registered. You will be informed about your complain within 24-hrs. For further asssitance please call";
        $url ='https://brandyourtext.com/sms/api/send?username=friends&password=123456&mask=FriendsCorp&mobile='.$mobile.'&message='.$message;


        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        $response = curl_exec ($ch);
        $err = curl_error($ch);  //if you need
        curl_close ($ch);
        return $response;*/
    }

}
