<?php

namespace App\Http\Controllers;


use App\Http\Models\Complaint;
use App\Http\Models\ComplaintStatus;
use App\Http\Models\User;
use App\Http\Models\Vendor;
use App\Http\Models\Corporate;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class CorporateController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $general['title']="All CRS";
        $general['msg']="Click on \"Edit\" to edit csr";
        $vendors=Vendor::all()->sortByDesc('id');
        return view('vendor.all_vendors',compact("general","vendors"));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $general['title']="Create CSR";
        $general['msg']="";
        return view('csr.add_csr',compact("general"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|unique:users',
            'username' => 'required',
            'password' => 'required|min:6',
            'phone_no' => 'required',
            'ptcl_no' => 'required',
            'address' => 'required',
        ]);
        $user= new User(
            $request->all()
        );
    
        $user->role_id=4;
        $user->isActive=1;
        $user->password=bcrypt($request->password);
        $user->username = $request->username;
        $user->save();
        $vendor=new Corporate(
            $request->all()
        );
        $vendor->user_id=$user->id;
        $vendor->save();
        session()->put('CSR Created', 'Successfully Created');
        return redirect(route('admin.csr.all_csr'));
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Vendor  $vendor
     * @return \Illuminate\Http\Response
     */
    public function show(Vendor $vendor)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Vendor  $vendor
     * @return \Illuminate\Http\Response
     */
    public function edit(Corporate $csr)
    {
        $general['title']="Edit CSR";
        $general['msg']="";
        return view('csr.edit',compact("general","csr"));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Vendor  $vendor
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Corporate $csr)
    {
        $validatedData = $request->validate([
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required',
            'username' => 'required',
            'password' => 'required',
            'phone_no' => 'required',
            'ptcl_no' => 'required',
            'address' => 'required',
        ]);
        $user=User::find($csr->user_id);
        $user->fill($request->all());
        $user->password =bcrypt($request->password);
        $user->username = $request->username;
        $user->email = $request->email;
        $user->save();
        $csr->fill($request->all());
        $csr->save();
        session()->put('companyCreated', 'Successfully Created');
        return redirect(route('admin.csr.all_csr'));
    }

    public function all_csr()
    {
       
        $general['title']="All CSR";
        $general['msg']="Click on \"Edit\" to edit csr";
        $csrs=Corporate::all()->sortByDesc('id');
        return view('csr.all_csrs',compact("general","csrs"));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Vendor  $vendor
     * @return \Illuminate\Http\Response
     */
    public function destroy(Corporate $csr)
    {
        $user=User::find($csr->user_id);
        $csr->delete();
        $user->delete();
        session()->put('vendorCreated', 'Successfully Deleted');
    }
    }
