<?php

namespace App\Http\Controllers;

use App\Http\Models\Company;
use App\Http\Models\Complaint;
use App\Http\Models\ComplaintStatus;
use App\Http\Models\Vendor;
use App\Http\Models\Supervisor;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Http\Models\User;


class SupervisorController extends Controller
{
    public function __construct()
    {
        //$this->middleware(['admin','supervisor']);
        //$this->middleware('admin');
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        
        $general['title']="Dashboard";
        $userid=Auth::id();
        $unassigned=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
            })
            ->where('status_id', '=', '2')
            ->count();
        $pending=Complaint::where('status_id', '!=', '5')->count();
        $completed=Complaint::where('status_id', '=', '5')->count();
        $uncompleted_forty_hours=Complaint::where('status_id', '!=', '5')
                    ->where('created_at','<',Carbon::now()->subHours(48))
                    ->count();
        $company_assigned=Complaint::where('status_id', '=', '3')->count();
        $vendor_pending=Complaint::whereIn('status_id',[4,8])->count(); 
        $non_updated_24_hours=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subHours(15))
        ->count();
        $non_updated_48_hours=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subHours(48))
        ->count();
        $non_updated_72_hours=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subHours(72))
        ->count();   
        $seven_days_pending=Complaint::where('status_id', '!=', '5')
        ->where('updated_at','<',Carbon::now()->subDays(7))
        ->count();
        $complaints_added_today=Complaint::wheredate('created_at','=',date('Y-m-d'))
            ->count();
        $complaints_completed_today=Complaint::where('status_id', '=', '5')
            ->wheredate('updated_at','=',date('Y-m-d'))
            ->count();    
        $company_pending=Complaint::with (['complaintstatus'])
                    ->whereHas('complaintstatus',function($q) {
                        $q->where(['status_id'=>3]);
                    })
                    ->where('status_id', '!=', '5')
                    ->count(); 
        $all_vendor_complaints=Complaint::with (['complaintstatus'])
                ->whereHas('complaintstatus',function($q) {
                    $q->whereIn('status_id',[4,8]);
                })
                ->count();      
        $reopened_pending=Complaint::where('status_id', '!=', '5')
                ->where('reopened','=',1)
                ->count();
        $reopened_completed=Complaint::where('status_id', '=', '5')
                ->where('reopened','=',1)
                ->count();
        $assigned=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
                $q->where('status_id', '=', '3')->orWhere('status_id', '=', '4');
            })
            ->where('status_id', '=', '3')->orWhere('status_id', '=', '4')
            ->count();
        /*$completed=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
            })
            ->where('status_id', '=', '5')
            ->count();*/
            
            $all_company_complaints=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) {
            $q->whereIn('status_id',[3]);
            })
            ->count();    
            
            $reassigned=Complaint::withAndWhereHas('complaintstatus',function($q) use ($userid) {
            $q->where(['assign_to' => $userid])->where('status_id' , '=' , 7)->whereIn('second_status_id' , [22,20]);
            })->where('status_id', '=', '7')
            ->count();
            
            $reassigned_48_hrs_passed=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
            $q->where(['assign_to'=>$userid]);
            })
            ->where('status_id', '=', '7')
            ->where('updated_at','<',Carbon::now()->subHours(48))
            ->count();
            
             $alertpending=Complaint::where('status_id', '!=', '5')->where('alert_complaint' , 1)->count();
            
        $happy_call_pending=Complaint::where('status_id','5')->whereNull('feedback')->count(); // feedback not added and complaint completed
        $happy_call_done=Complaint::where('status_id','5')->whereNotNull('feedback')->count(); // feedback added and complaint completed
        
        return view('supervisor.dashboard',compact("alertpending" , 'happy_call_pending' , 'happy_call_done' , "general","pending","unassigned","all_company_complaints","all_vendor_complaints","company_pending","company_assigned","seven_days_pending","non_updated_24_hours","non_updated_48_hours","vendor_pending","non_updated_72_hours","uncompleted_forty_hours","reassigned_48_hrs_passed","assigned","completed","reassigned","complaints_completed_today","complaints_added_today","reopened_completed","reopened_pending"));
    }

    public function pending(Request $request)
    {
        $general['title'] = "Unassigned Complaints";
        $general['selected_record'] = "all pending";
        $userid = Auth::id();
        $com=Complaint::query();
        if($request->has('pending_duration') && $request->input('pending_duration')=="more_than_48_hrs"){
            $com->where('updated_at','<',Carbon::now()->subHours(48));  // more than 48 hrs passed
            $general['selected_record'] = "more_than_48_hrs";
        }
        $complaints = $com->with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
            })
            //->join('users', 'users.id','=','complaints.created_by')
            ->where('status_id', '=', '2')
            ->get()->sortByDesc('id');
          //  dd($complaints);
        $vendors=Vendor::all()->sortByDesc('id');
        $companies=Company::all()->sortByDesc('id');

        return view('supervisor.pending',compact("general","complaints","companies","vendors"));
    }

    

    public function assigned()
    {
        $general['title'] = "Assigned Complaints";
        $general['msg'] = "Assigned Complaints";
        $userid = Auth::id();
        $complaints = Complaint::with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->where('status_id', '=', '3')->orWhere('status_id', '=', '4');
            })
            ->where('status_id', '=', '3')->orWhere('status_id', '=', '4')
            ->get()->sortByDesc('id');

        return view('supervisor.assigned',compact("general","complaints"));
    }
    public function notCompleted()
    {
        $general['title'] = "Not completed Complaints";
        $general['msg'] = "Red alert Complaints";
        $userid = Auth::id();
        $complaints = Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
            })
            ->where('status_id', '!=', '5')
            ->where('created_at','<',Carbon::now()->subDays(7))
            ->get()->sortByDesc('id');

        return view('supervisor.completed',compact("general","complaints"));
    }
    public function completed()
    {
        $general['title'] = "Completed Complaints";
        $general['msg'] = "Completed Complaints";
        $userid = Auth::id();
        $complaints = Complaint::with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->where('status_id', '=', '2');
            })
            ->where('status_id', '=', '5')
            ->get()->sortByDesc('id');

        return view('supervisor.completed',compact("general","complaints"));
    }
    public function reassigned(Request $request)
    {
        $general['title'] = "Reassigned Complaints";
        $general['selected_record'] = "all_reassigned";
        $userid = Auth::id();
        $com=Complaint::query();
        if($request->has('pending_duration') && $request->input('pending_duration')=="more_than_48_hrs"){
            $com->where('updated_at','<',Carbon::now()->subHours(48));  // more than 48 hrs passed
            $general['selected_record'] = "more_than_48_hrs";
        }
        $complaints = $com->withAndWhereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->where('status_id', '=', '7');
                $q->whereIn('second_status_id' , [22,20]);
            })
            ->where('status_id', '=', '7')
            ->get()->sortByDesc('id');
            
         

        return view('supervisor.reassigned',compact("general","complaints"));
    }
    public function assign(Request $request)
    {
        $complaint=Complaint::find($request->input('id'));
        if ($complaint){
            $status= new ComplaintStatus();
            if ($request->input('assignTo')=="company"){
                if($request->has('company_id') && $request->filled('company_id')){
                    $status->status_id=3;
                    $status->assign_to=$request->input('company_id');
                } else{
                    return ['status'=>'failure','msg'=>'select comapany'];
                }
                
            }else if ($request->input('assignTo')=="vendor"){
                if($request->has('vendor_id') && $request->filled('vendor_id')){
                    $status->status_id=4;
                    $status->assign_to=$request->input('vendor_id');
                } else{
                    return ['status'=>'failure','msg'=>'select vendor'];
                }
            }else{
                $status->status_id=16;
                $status->assign_to=1;
                $status->comment=$request->input('comment');
            }
            $status->complaint_id=$complaint->id;
            $status->save();

            session()->put('assigned', 'Successfully Assigned');
            return ['status'=>'success','msg'=>'Error in Assigning'];
        }
        return ['status'=>'failure','msg'=>'Error in Assigning'];

    }
    public function ComplaintView($id){
       
        $general['title']="Complaint";
        $general['msg']="Click on ";
        $complaint = Complaint::find($id);
        return view('supervisor.complaint_view',compact("general","complaint"));
    }
    function ComplaintViewPost(Request $request,$id){
       
        $complaint = Complaint::find($id);
        if ($complaint->status_id==7){
            if ($request->input('completed')){
                $assignTo=NULL;
                $status_id=5;
            }else{
                $status = ComplaintStatus::where(['status_id' => 4, 'complaint_id' => $id])->first();
                $assignTo=$status->assign_to;
                $status_id=8;
            }
            $comment = $request->input('comment');
            $second_status = $request->input('status_id');
            $cstatus = new ComplaintStatus(['comment' => $comment, 'status_id' => $status_id, 'second_status_id' => $second_status, 'assign_to' => $assignTo]);
            $complaint = Complaint::find($id);
            $complaint->status_id = $status_id;
            $complaint->complaintstatus()->save($cstatus);
            session()->flash('success', 'Updated');
        }
        return redirect(route('supervisor.complaint.view',$id));
    }
    
    
        function ComplaintReassignPost(Request $request,$id){
        
        $complaint = Complaint::find($id);
        if ($complaint->status_id==7)
        {
        if ($request->input('status') == 'completed'){
        $assignTo=NULL;
        $status_id=5;
        }else{
        $status = ComplaintStatus::where(['status_id' => 4, 'complaint_id' => $id])->first();
        $assignTo=$status->assign_to;
        $status_id=8;
        }
        
        $comment = $request->input('comment');
        $second_status = $request->input('status_id');
        $cstatus = new ComplaintStatus(['comment' => $comment, 'status_id' => $status_id, 'second_status_id' => $second_status, 'assign_to' => $assignTo]);
        $complaint->status_id = $status_id;
        $complaint->complaintstatus()->save($cstatus);
        
        echo json_encode(array('status' => 'success' , 'msg' => 'Updated'));
        
        }
        else
        {
        echo json_encode(array('status' => 'failure' , 'msg' => 'Error!'));
        }
        
        }
    

    function specialRequest(){
        $general['title']="Special Request";
        $general['msg']=" ";
        $userid = Auth::id();
        $complaints = Complaint::with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
            })
            ->where('status_id', '=', '2')
            ->get()->sortByDesc('id');

        return view('supervisor.specialRequest',compact("general","complaints"));
    }

    public function create()
    {
        $general['title']="Create Supervisor";
        $general['msg']="";
        return view('supervisor.add_supervisor',compact("general"));
    }

    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|unique:users',
            'username' => 'required',
            'password' => 'required|min:6',
            'phone_no' => 'required',
            'ptcl_no' => 'required',
            'address' => 'required',
        ]);
        $user= new User(
            $request->all()
        );
        $user->role_id=3;
        $user->isActive=1;
        $user->password=bcrypt($request->password);
        $user->username = $request->username;
        $user->save();
        $vendor=new Supervisor(
            $request->all()
        );
        $vendor->user_id=$user->id;
        $vendor->save();
        session()->put('SupervisorCreated', 'Successfully Created');
        return redirect(route('admin.supervisor.all_supervisor'));
    }

    public function all_supervisor()
    {
       
        $general['title']="All Supervisors";
        $general['msg']="Click on \"Edit\" to edit supervisor";
        $supervisors=Supervisor::all()->sortByDesc('id');
        return view('supervisor.all_supervisors',compact("general","supervisors"));
    }


    public function edit(Supervisor $supervisor)
    {
        $general['title']="Edit Supervisor";
        $general['msg']="";
        return view('supervisor.edit',compact("general","supervisor"));
    }

    public function update(Request $request, Supervisor $supervisor)
    {
        $validatedData = $request->validate([
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required',
            'username' => 'required',
            'password' => 'required',
            'phone_no' => 'required',
            'ptcl_no' => 'required',
            'address' => 'required',
        ]);
        $user=User::find($supervisor->user_id);
        $user->fill($request->all());
        $user->password =bcrypt($request->password);
        $user->email = $request->email;
        $user->username = $request->username;
        $user->save();
        $supervisor->fill($request->all());
        $supervisor->save();
        session()->put('companyCreated', 'Successfully Created');
        return redirect(route('admin.supervisor.all_supervisor'));
    }

    public function destroy(Supervisor $supervisor)
    {
        $user=User::find($supervisor->user_id);
        $supervisor->delete();
        $user->delete();
        session()->put('vendorCreated', 'Successfully Deleted');
    }
}
