<?php

namespace App\Http\Controllers;


use App\Http\Models\Complaint;
use App\Http\Models\ComplaintStatus;
use App\Http\Models\User;
use App\Http\Models\Vendor;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class VendorController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $general['title']="All Vendors";
        $general['msg']="Click on \"Edit\" to edit vendor";
        $vendors=Vendor::all()->sortByDesc('id');
        return view('vendor.all_vendors',compact("general","vendors"));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $general['title']="Create Vendor";
        $general['msg']="";
        return view('vendor.create',compact("general"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|unique:users',
            'username' => 'required',
            'password' => 'required|min:6',
            'phone_no' => 'required',
            'ptcl_no' => 'required',
            'address' => 'required',
        ]);
        $user= new User(
            $request->all()
        );
        $user->role_id=6;
        $user->isActive=1;
        $user->password=bcrypt($request->password);
        $user->email = $request->email;
        $user->username = $request->username;
        $user->save();
        $vendor=new Vendor(
            $request->all()
        );
        $vendor->user_id=$user->id;
        $vendor->save();
        session()->put('VendorCreated', 'Successfully Created');
        return redirect(route('admin.vendor.index'));
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Vendor  $vendor
     * @return \Illuminate\Http\Response
     */
    public function show(Vendor $vendor)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Vendor  $vendor
     * @return \Illuminate\Http\Response
     */
    public function edit(Vendor $vendor)
    {
        $general['title']="Edit Vendor";
        $general['msg']="";
        return view('vendor.edit',compact("general","vendor"));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Vendor  $vendor
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Vendor $vendor)
    {
        $validatedData = $request->validate([
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required',
            'username' => 'required',
            'password' => 'required',
            'phone_no' => 'required',
            'ptcl_no' => 'required',
            'address' => 'required',
        ]);
        $user=User::find($vendor->user_id);
        $user->fill($request->all());
        $user->password=bcrypt($request->password);
        $user->username = $request->username;
        $user->email = $request->email;
        $user->save();
        $vendor->fill($request->all());
        $vendor->save();
        session()->put('companyCreated', 'Successfully Created');
        return redirect(route('admin.vendor.index'));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Vendor  $vendor
     * @return \Illuminate\Http\Response
     */
    public function destroy(Vendor $vendor)
    {
        $user=User::find($vendor->user_id);
        $vendor->delete();
        $user->delete();
        session()->put('vendorCreated', 'Successfully Deleted');
    }
    public function vendor_dashboard()
    {
        $general['title']="Dashboard";
        $userid=Auth::user()->vendor->id;
       /* $pending=Complaint::with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->whereIn('status_id',[4,8]);
            })
            ->whereIn('status_id',[4,8])
            ->count();*/
        
        /*$assigned=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
            })
            ->where('status_id', '=', '3')->orWhere('status_id', '=', '4')
            ->count();*/
        $completed=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
                $q->whereIN('status_id',[4,8]);
            })
            ->where('status_id', '=', '5')
            ->count();
        $total_complaints=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
                $q->whereIN('status_id',[4,8]);
            })
            ->count(); 
        /*$reassigned=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
            })
            ->where('status_id', '=', '8')
            ->count();*/
        $notResolved=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
                $q->whereIN('status_id',[4,8]);
            })
            ->where('status_id', '!=', '5')
           // ->where('created_at','<',Carbon::now()->subDays(15))
            ->count();
        $passed_2_days=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
                $q->whereIN('status_id',[4,8]);
            })
            ->where('status_id', '!=', '5')
            ->where('created_at','<',Carbon::now()->subDays(2))
            ->count();
        $passed_4_days=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
                $q->whereIN('status_id',[4,8]);
            })
            ->where('status_id', '!=', '5')
            ->where('created_at','<',Carbon::now()->subDays(4))
            ->count(); 
        $passed_8_days=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
                $q->whereIN('status_id',[4,8]);
            })
            ->where('status_id', '!=', '5')
            ->where('created_at','<',Carbon::now()->subDays(8))
            ->count();
            $passed_12_days=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid]);
                $q->whereIN('status_id',[4,8]);
            })
            ->where('status_id', '!=', '5')
            ->where('created_at','<',Carbon::now()->subDays(12))
            ->count();                  
         $user_id_orignal=Auth::id();
        $assigned_backed=Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($user_id_orignal) {
                $q->where(['created_by'=>$user_id_orignal]);
            })
            ->where('status_id', '=', '7')
            ->count();   
        return view('vendor.dashboard',compact("general","completed","total_complaints","notResolved",'assigned_backed',"passed_2_days","passed_4_days","passed_8_days","passed_12_days"));
    }
    function vendor_complaints(Request $request){
        $general['title']="Your Complaints";
        $general['selected_record']="all_pending";
        $userid = Auth::user()->vendor->id;
        $com=Complaint::query();
        if($request->has('pending_duration') && $request->input('pending_duration')=="more_than_48_hrs"){
            $com->where('updated_at','<',Carbon::now()->subHours(48));  // more than 48 hrs passed
            $general['selected_record'] = "more_than_48_hrs";
        }
        $complaints = $com->with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->where('status_id', '=', '4');
            })
            ->where('status_id', '=', '4')
            ->get()->sortByDesc('id');
        return view('vendor.new_complaints',compact("general","complaints"));
    }
    /****all vedr complaints */
    function vendor_all_complaints()
    {
        $general['title']="Your Complaints";
        $general['msg']="all complaints";
        $userid = Auth::user()->vendor->id;
        $complaints = Complaint::with (['complaintstatus'])
        ->whereHas('complaintstatus',function($q) use ($userid) {
            $q->where(['assign_to'=>$userid]);
            $q->whereIN('status_id',[4,8]);
        })->get()->sortByDesc('id');
        return view('vendor.all_complaints',compact("general","complaints"));
    }

    function vendor_reassigned_complaints(){
        $general['title']="Reassigned Complaints";
        $general['msg']="Click on \"View\" to View complaint";
        $userid = Auth::user()->vendor->id;
        $complaints = Complaint::with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->where('status_id', '=', '8');
            })
            ->where('status_id', '=', '8')
            ->get();
        return view('vendor.reassigned_complaints',compact("general","complaints"));
    }

    function vendor_completed_complaints(){
        $general['title']="Completed Complaints";
        $general['msg']="Click on \"View\" to View complaint";
        $userid = Auth::user()->vendor->id;
        $complaints = Complaint::with(['complaintstatus'])
            ->whereHas('complaintstatus', function ($q) use ($userid) {
                $q->where(['assign_to' => $userid]);
                $q->whereIN('status_id',[4,8]);
            })
            ->where('status_id', '=', '5')
            ->get();
        return view('vendor.reassigned_complaints',compact("general","complaints"));
    }
    function vendor_notCompleted_complaints(){
        $general['title']="Pending Complaints";
        $general['msg']="";
        $userid = Auth::user()->vendor->id;
        $complaints = Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['assign_to'=>$userid,]);
                $q->whereIN('status_id',[4,8]);
            })
            ->where('status_id', '!=', '5')
         //   ->where('created_at','<',Carbon::now()->subDays(15))
            ->get();
        return view('vendor.reassigned_complaints',compact("general","complaints"));
    }
    /***
     * passed ? days and not completed
     */
    function pendingfromdays($days=2)
    {
        $general['title']="Passed $days days Pending";
        $general['msg']="";
        $userid = Auth::user()->vendor->id;
        $complaints = Complaint::with (['complaintstatus'])
        ->whereHas('complaintstatus',function($q) use ($userid) {
            $q->where(['assign_to'=>$userid]);
            $q->whereIN('status_id',[4,8]);
        })
        ->where('status_id', '!=', '5')
        ->where('created_at','<',Carbon::now()->subDays($days))
            ->get();
        return view('vendor.all_complaints',compact("general","complaints"));
    }
    /**
     * asigned back to supervisor by vendor
     */
    function assigned_back(){
        $general['title']="Assigned back to supervisor";
        $general['msg']="";
        $userid = Auth::id();
        $complaints = Complaint::with (['complaintstatus'])
            ->whereHas('complaintstatus',function($q) use ($userid) {
                $q->where(['created_by'=>$userid]);
            })
            ->where('status_id', '=', '7')
            ->get();
        return view('vendor.assigned_back_to_supervisor_complaints',compact("general","complaints"));
    }
    function vendor_complaint_view($id){
        
        $general['title']="Complaint";
        $general['msg']="Click on ";
        $userid = Auth::user()->vendor->id;
        $complaint = Complaint::find($id);
        
        if($complaint->view_status == 0 )
        {
           Complaint::where('id', $complaint->id)
            ->update(['view_status' => '1']); 
        }
        
        return view('vendor.complaint_view',compact("general","complaint"));
    }
    function vendor_complaint_view_post(Request $request,$id){
        
        if(empty($request->status_id) || empty($request->comment))
        {
            return redirect(route('vendor.complaint.view' , [$id]))->with('error' , 'Both Status and Note Fields Required.');
        }
        
        $complaint = Complaint::find($id);
        if ($complaint->status_id==4 || $complaint->status_id==8) {
            $comment = $request->input('comment');
            $second_status = $request->input('status_id');
            $status = ComplaintStatus::where(['status_id' => 2, 'complaint_id' => $id])->first();
            $cstatus = new ComplaintStatus(['comment' => $comment, 'status_id' => 7,'second_status_id' => $second_status, 'assign_to' => $status->assign_to]);
            $complaint = Complaint::find($id);
            $complaint->status_id = 7;
            $complaint->complaintstatus()->save($cstatus);
        }
        return redirect(route('vendor'));
    }
}
